#include "bleprofile.h"
#include "bleapp.h"
#include "platform.h"

#include "gpiodriver.h"
#include "pwm.h"

#define LED_RED    26  /* PWM 0 */
#define LED_BLUE   27  /* PWM 1 */
#define LED_GREEN  28  /* PWM 2 */

#define GPIO_PORT(x)  (x/16)
#define GPIO_PIN(x)   (x%16)

#define ENABLE_LED_RED    PWM0_OUTPUT_ENABLE_P26
#define ENABLE_LED_BLUE   PWM1_OUTPUT_ENABLE_P27
#define ENABLE_LED_GREEN  PWM2_OUTPUT_ENABLE_P28

#define DISABLE_LED_GPIO     GPIO_INPUT_ENABLE | GPIO_PULL_DOWN
#define LED_RED_CHANNEL      (PWM0)
#define LED_BLUE_CHANNEL     (PWM1)
#define LED_GREEN_CHANNEL    (PWM2)


#define FULL_PWM             (0x000)
#define HALF_PWM             (0x210)

typedef enum {
	RED = 0,
	BLUE,
	GREEN
} colour;

static void basic_create(void);
static void basic_interrupt_handler (void* parameter, UINT8 arg);

static colour current_colour = RED;

const UINT8 basic_gatt_database[]= {};

const BLE_PROFILE_CFG basic_cfg = {};

const BLE_PROFILE_PUART_CFG basic_puart_cfg = {
	/*.baudrate   =*/ 115200,
	/*.txpin      =*/ PUARTDISABLE | GPIO_PIN_UART_TX,
	/*.rxpin      =*/ PUARTDISABLE | GPIO_PIN_UART_RX,
};

const BLE_PROFILE_GPIO_CFG basic_gpio_cfg = {};

// Following structure defines GPIO configuration used by the application
const BLE_PROFILE_GPIO_CFG hello_sensor_gpio_cfg = {};


APPLICATION_INIT() {
    bleapp_set_cfg(NULL,
       				0,
       				NULL,
       				(void *) &basic_puart_cfg,
       				NULL,
       				basic_create);
}

void basic_create(void)
{

    bleprofile_Init(bleprofile_p_cfg);
    bleprofile_GPIOInit(bleprofile_gpio_p_cfg);

    // Enable Button Interrupt, press SW2 of Tag4 to change the LED brightness
    UINT16 masks[3] = {(1 << 0), 0 , 0};
    gpio_registerForInterrupt(masks, basic_interrupt_handler, NULL);
    gpio_configurePin(0, 0, GPIO_EN_INT_RISING_EDGE, GPIO_PIN_OUTPUT_LOW);

    //gpio_configurePin((GPIO_PIN_P28)/16, (GPIO_PIN_P28)%16, GPIO_INPUT_ENABLE, 0);

	// Enable LED
	gpio_configurePin(GPIO_PORT(LED_BLUE), GPIO_PIN(LED_BLUE), ENABLE_LED_BLUE, 0);
	pwm_start( LED_BLUE_CHANNEL, LHL_CLK, 0x000 , 0);

	// pwm_start( id, clk, toggleCount, initCount);
	// Output Voltage | Toggle Count
	// -----------------------------
	// ~ (4/4) V_MAX  |    0x000
	// ~ (3/4) V_MAX  |    0x100
	// ~ (2/4) V_MAX  |    0x210
	// ~ (1/4) V_MAX  |    0x315
}

void basic_interrupt_handler (void* parameter, UINT8 arg)
{
	// wrap around current_state
	if(++current_colour > 2)
		current_colour = 0;

	switch(current_colour) {
	case RED:
		gpio_configurePin(GPIO_PORT(LED_RED), GPIO_PIN(LED_RED), DISABLE_LED_GPIO, 0);
		pwm_disableChannel( 1 << LED_RED_CHANNEL );
		gpio_configurePin(GPIO_PORT(LED_BLUE), GPIO_PIN(LED_BLUE), ENABLE_LED_BLUE, 0);
		pwm_start( LED_BLUE_CHANNEL, LHL_CLK, FULL_PWM , 0);
		break;

	case BLUE:
		gpio_configurePin(GPIO_PORT(LED_BLUE), GPIO_PIN(LED_BLUE), DISABLE_LED_GPIO, 0);
	    pwm_disableChannel( 1 << LED_BLUE_CHANNEL );
		gpio_configurePin(GPIO_PORT(LED_GREEN), GPIO_PIN(LED_GREEN), ENABLE_LED_GREEN, 0);
		pwm_start( LED_GREEN_CHANNEL, LHL_CLK, FULL_PWM , 0);
		break;

	case GREEN:
		gpio_configurePin(GPIO_PORT(LED_GREEN), GPIO_PIN(LED_GREEN), DISABLE_LED_GPIO, 0);
		pwm_disableChannel( 1 << LED_GREEN_CHANNEL );
		gpio_configurePin(GPIO_PORT(LED_RED), GPIO_PIN(LED_RED), ENABLE_LED_RED, 0);
		pwm_start( LED_RED_CHANNEL, LHL_CLK, FULL_PWM , 0);
		break;
	}


}












